# wxSU
# Copyright (c) 2008-2009 Alliance for Sustainable Energy.  All rights reserved.
# See the file "License.txt" for additional terms and conditions.

require("wxSU/lib/wxruby/lib/wx")
require("wxSU/ext/Win32API")


module WxSU

  class MSWindowsFrame < Wx::Frame

    def initialize
      title = "wxSU"

      # Create a dummy Wx::Frame off screen
      super(nil, -1, title, Wx::Point.new(-50,-50), Wx::Size.new(0,0), Wx::FRAME_NO_TASKBAR)

      # Show the window and grab its handle
      show
      findWindow = Win32API.new("user32.dll", "FindWindow", ['P','P'], 'N')
      surrogate_window_id = findWindow.call(0, title)

      # Set locale specific strings
      locale = Sketchup.get_locale

      if (locale =~ /en/)
        untitled = "Untitled"  # English
      elsif (locale =~ /ja/)
        untitled = "Untitled"  # Japanese - Don't know how to handle Japanese characters!
      elsif (locale =~ /it/)
        untitled = "Senza titolo"  # Italian
      elsif (locale =~ /fr/)
        untitled = "Sans titre"  # French
      elsif (locale =~ /es/)
        untitled = "Sin ttulo"  # Spanish
      elsif (locale =~ /de/)
        untitled = "Unbenannt"  # German
      else
        untitled = "Untitled"
      end

      # These strings are reportedly not translated
      sketchup_free = "SketchUp"
      sketchup_pro = "SketchUp Pro"
      no_license = "[LICENSE UNAVAILABLE]"

      # Find the SketchUp main window handle by its window title
      model_path = Sketchup.active_model.path
      if (model_path.empty?)
        model_name = untitled

        if (Sketchup.app_name == "Google SketchUp")
          sketchup_title = model_name + " - " + sketchup_free
        else
          sketchup_title = model_name + " - " + sketchup_pro
        end
      else
        # For some reason, when starting SketchUp by double-clicking an SKP file, the window title never says SketchUp Pro.
	model_name=File.basename(model_path,".skp") #Fix by TM
        sketchup_title = model_name + " - " + sketchup_free
      end

      @window_id = findWindow.call(0, sketchup_title)

      if (@window_id == 0)
        # Can't find the window, look for no license version
        @window_id = findWindow.call(0, sketchup_title + " " + no_license)
      end

      #UI.messagebox("SketchUp window name #{(@window_id == 0 ? 'not' : '')} recognised\nlocale: #{locale}\nSketchup_title: #{sketchup_title}",MB_OK)

      if (@window_id > 0)
        setParent = Win32API.new("user32.dll", "SetParent", ['N', 'N'], 'N')
        setParent.call(surrogate_window_id, @window_id)
      end
    end


    def get_rect
      if (@window_id > 0)
        window_id = @window_id
      else
        # Couldn't find the SketchUp window, use the desktop window
        # This is most useful for center_on_parent
        getDesktop = Win32API.new("user32.dll", "GetDesktopWindow", [], 'N')
        window_id = getDesktop.call
      end

      getWindowRect = Win32API.new("user32.dll", "GetWindowRect", ['P', 'P'], 'N')

      # Create a char buffer large enough for the rectangle (4 integers)
      rect = Array.new.fill(0.chr, 0..4*4).join
      getWindowRect.call(window_id, rect)
      # Turn char buffer into an array of integers
      rect = rect.unpack("i*")

      # Turn rectangle into x, y, w, h
      width = rect[2] - rect[0]  # w = x2 - x1
      height = rect[3] - rect[1]  # h = y2 - y1

      return(Wx::Rect.new(rect[0], rect[1], width, height))
    end


    def get_size
      rect = get_rect()
      return(Wx::Size.new(rect.width, rect.height))
    end


    def get_position
      rect = get_rect()
      return(Wx::Point.new(rect.x, rect.y))
    end


    def set_size(value)
      rect = get_rect()
      setWindowPos = Win32API.new("user32.dll", "SetWindowPos", ['P','P','N','N','N','N','N'], 'N')

      if (value.class == Wx::Rect)
        setWindowPos.call(@window_id, 0, value.x, value.y, value.x + value.width, value.y + value.height, 0)
      elsif (value.class == Wx::Size)
        setWindowPos.call(@window_id, 0, rect.x, rect.y, rect.x + value.get_width, rect.y + value.get_height, 0)
      end
    end


    def raise
      setForeground = Win32API.new("user32.dll", "SetForegroundWindow", ['P'], 'N')
      setForeground.call(@window_id)
    end


    def disable
      getDesktop = Win32API.new("user32.dll", "GetDesktopWindow", [], 'N')
      desktop_window_id = getDesktop.call

      setForeground = Win32API.new("user32.dll", "SetForegroundWindow", ['P'], 'N')
      setForeground.call(desktop_window_id)
    end


    # This method causes SketchUp to exit.
    def close
      sendMessage = Win32API.new("user32.dll", "SendMessage", ['N','N','N','P'], 'N')
      # Send a "WM_CLOSE" message (0x0010)
      sendMessage.call(@window_id, 0x0010, 0, "")
    end

  end

end
